/**
 * Service
 * @class ApEventsourceService
 * @param {Object} store - Redux store
 * @param {Object} options - Optional settings
 */
'use strict'

const { ApService, operations } = require('apeman-service-base')
const abind = require('abind')
const EventSource = require('eventsource')

/** @lends ApEventsourceService */
class ApEventsourceService extends ApService {
  constructor (store, options = {}) {
    super(store)
    const s = this
    abind(s, { proto: ApEventsourceService.prototype })
    abind(s)
    let {
      scopeName = 'eventsource'
    } = options
    s.scopeName = scopeName
  }

  /**
   * Observe event source
   * @param {string} url
   * @param {Object} params
   * @param {Object} handlers - Event handlers
   * @returns {EventSource}
   */
  observe (url, params, handlers = {}) {
    let eventSource = new EventSource(url, params)
    for (let event of Object.keys(handlers)) {
      let handler = handlers[ event ]
      eventSource.on(event, ({ data }) => {
        try {
          data = JSON.parse(data)
        } catch (e) {
          // Do nothing
        }
        handler(data)
      })
    }
    return eventSource
  }
}

module.exports = ApEventsourceService
